      !-----------------------------------------------------------------
	!
      !  Purpose: To read and load EGM 2008 gravity model (available at
	!          "http://earth-info.nga.mil/GandG/wgs84/gravitymod/
	!           egm2008/first_release.html" 
	!           with no separation spaces between slashes)
      !
      ! Input:
      !      NM    I2 desired order and degree of model (max = 2190)
      !
	! Outputs:
	!      GM    R8 Geogravitational constant [m**3/s**2]
	!      AE    R8 Earth equatorial radius [m]
	!       C    R8 Array (0:NMAX,0:Nm) Harmonic coefficients C(n,m)
	!       S    R8 Array (0:NMAX,0:Nm) Harmonic coefficients S(n,m)
	!
      ! Author: Helio Koiti Kuga Jan-2007 Fortran Version 1.0 Created
	!
      ! Refs.: 
	!    Pavlis, N.K.; Holmes, S.A.; Kenyon, S.C.; Factor, J.K. "An Earth 
	!      Gravitational Model to Degree2160: EGM2008," presented at 
	!      the 2008 General Assembly of the European Geosciences Union, 
	!      Vienna, Austria, April 13-18, 2008.
	!
      !-----------------------------------------------------------------

      SUBROUTINE READ_EGM2008 (NM, GM, AE, C, S)
      IMPLICIT NONE

      INTEGER*2 NM
      REAL*8 GM, AE
	REAL*8  C(0:NM,0:NM), S(0:NM,0:NM)   ! Load up to NM order/degree

	INTEGER*2, PARAMETER :: iunit = 13,  ! Check if it is in use elsewhere
     *                        NMAX = 2190  ! Max order/degree of model.
     *                                     ! Currently highest is EGM 2008!
     *                                     ! Change here if your model is higher
      INTEGER*2 N, M
	REAL*8 CNM, SNM

      ! Load file of geopotential model 
      open (iunit, file='EGM2008_ZeroTide_truncated.dat', status='old')

      ! Define constants of model: GM and Mean Earth radius (currently EGM2008)
      GM =    3986004.415D+8     ! [m^3/s^2]
      AE =    6378136.3D0        ! [m]

      ! Loop for reading and loading harmonic coefficients
      m = 0
      do while (m < Nm) 
        read (iunit, *) n, m, cnm, snm  ! Free format read
        if (n > Nmax) exit
	  C(n,m) = cnm
	  S(n,m) = snm
      end do

      ! Close file
      close (iunit)

      return
	end